@@ -29,18 +29,18 @@
 #' \describe{
 #'     \item{\code{fit}}{Fitting set.}
 #'     \item{\code{test}}{Test set.}
-#'     \item{\code{feature_selection}}{Logical vector indicating which features were kept
-#'         (TRUE) and discarded (FALSE).}
+#'     \item{\code{feature_selection}}{Integer vector mapping the features of
+#'           the training and test sets to the original data sets.}
 #'     \item{\code{fold}}{The fold that was used to split the data.}
 #' }
 #'
 #' @example examples/pre-process.r
-#' @seealso \code{\link{emil}}, \code{\link{pre_impute_knn}}
+#' @seealso \code{\link{pre_factor_to_logical}}, \code{\link{emil}},
+#'   \code{\link{pre_impute_knn}}
 #' @author Christofer \enc{Bäcklin}{Backlin}
 #' @name pre_process
 {}
 
-
 #' @param x Dataset.
 #' @param y Response vector.
 #' @param fold A logical or numeric vector with \code{TRUE} or positive numbers
@@ -66,12 +66,44 @@ pre_split <- function(x, y, fold){
                         y = y[index_fit(fold)]),
              test = list(x = x[index_test(fold),,drop=FALSE],
                          y = y[index_test(fold)]),
-             feature_selection = structure(rep(TRUE, ncol(x)),
-                                           names = colnames(x)),
+             feature_selection = structure(1:ncol(x), names = colnames(x)),
              fold = fold),
         class=c("preprocessed_data", "list"))
 }
 
+#' Validate a pre-processed data set
+#' 
+#' While writing and debugging pre-processing functions this function can be
+#' useful to confirm that the resulting data sets fulfills the necessary
+#' requirements.
+#' 
+#' @param data Pre-processed data set.
+#' @return Nothing, only throws an error or prints a completion message.
+#' @author Christofer \enc{Bäcklin}{Backlin}
+#' @export 
+validate_data <- function(data){
+    stopifnot(inherits(data, "preprocessed_data"))
+    stopifnot(identical(colnames(data$fit$x), colnames(data$test$x)))
+    stopifnot(identical(colnames(data$fit$x), names(data$feature_selection)))
+    cat("Data set is valid.")
+}
+
+#' Print method for pre-processed data
+#' 
+#' @method print preprocessed_data
+#' @param x Pre-processed data, as produced by \code{\link{pre_split}}.
+#' @param ... Ignored, kept for S3 consistency.
+#' @return Nothing
+#' @author Christofer \enc{Bäcklin}{Backlin}
+#' @export
+print.preprocessed_data <- function(x, ...){
+    feature <- table(factor(x$feature_selection, c(FALSE, TRUE)))
+    cat("Pre-processed data set `", x$name, "` of ",
+        ncol(x$fit$x), " features\n",
+        nrow(x$fit$x), " observations for model fitting,\n",
+        nrow(x$test$x), " observations for model evaluation.\n", sep="")
+}
+
 #' @param data Fitting and testing data sets, as returned by
 #'   \code{\link{pre_split}}.
 #' @param x_fun Function to apply to the descriptors of the datasets
@@ -115,7 +147,9 @@ pre_remove <- function(data, feature){
     }
     data$fit$x <- data$fit$x[, -feature, drop=FALSE]
     data$test$x <- data$test$x[, -feature, drop=FALSE]
-    data$feature_selection[feature] <- FALSE
+    data$feature_selection <- data$feature_selection[-feature]
+    stopifnot(ncol(data$fit$x) == length(data$feature_selection),
+              ncol(data$test$x) == length(data$feature_selection))
     data
 }
 
@@ -154,8 +188,15 @@ pre_scale <- function(data, y=FALSE, na.rm=TRUE, center=TRUE){
 
 #' @rdname pre_process
 #' @export
-pre_remove_constant <- function(data){
-    pre_remove(data, apply(data$fit$x, 2, sd) == 0)
+pre_remove_constant <- function(data, na.rm=TRUE){
+    if(is.data.frame(data$fit$x)){
+        constant_feature <- vapply(data$fit$x, is_constant, logical(1), na.rm=na.rm)
+    } else constant_feature <- apply(data$fit$x, 2, is_constant, na.rm=na.rm)
+    if(has_na(constant_feature)){
+        if(na.rm) constant_feature[is.na(constant_feature)] <- TRUE
+        else stop("Could not determine which features are constant.")
+    } 
+    pre_remove(data, constant_feature)
 }
 
 #' @param cutoff See \code{\link[caret]{findCorrelation}}.
@@ -185,210 +226,93 @@ pre_pca <- function(data, ncomponent, scale. = TRUE, ...){
     data
 }
 
-#' Support function for identifying missing values
+#' Convert factors to logical columns
 #' 
-#' @param data Fitting and testing data sets, as returned by
-#'   \code{\link{pre_split}}.
-#' @return Data frame containing row and column indices of missing values or
-#'   \code{NULL} if the data doesn't contain any.
-#' @author Christofer \enc{Bäcklin}{Backlin}
-#' @export
-na_index <- function(data){
-    # `unname` is needed to avoid problems with duplicate names
-    fit.na  <- which(is.na(unname(data$fit$x)), arr.ind=TRUE)
-    test.na <- which(is.na(unname(data$test$x)), arr.ind=TRUE)
-    rbind(
-        if(any(fit.na)) data.frame(set = "fit", fit.na) else NULL,
-        if(any(test.na)) data.frame(set = "test", test.na) else NULL
-    )
-}
-
-#' Basic imputation
-#'
-#' This solution is optimized for the scenario that the dataset is very large
-#' but only contains missing values in a small number of columns.
-#'
-#' @param data Fitting and test datasets, as returned by \code{\link{pre_split}}
-#'   or any other standard pre-processing function.
-#' @param fun Function for calculating imputation values. Should take a vector
-#'  and return a scalar.
-#' @param ... Sent to \code{fun}.
-#' @return A pair of fitting and testing datasets.
-#' @author Christofer \enc{Bäcklin}{Backlin}
-#' @export
-pre_impute <- function(data, fun, ...){
-    na.ind <- na_index(data)
-    if(is.null(na.ind)) return(data)
-
-    na.feats <- unique(na.ind$col)
-    m <- rep(NA, ncol(data$fit$x))
-    m[na.feats] <- apply(data$fit$x[,na.feats,drop=FALSE], 2, fun, ...)
-
-    na.ind %<>%
-        split(na.ind$set) %>%
-        lapply(function(x) as.matrix(x[-1]))
-    if(!is.null(na.ind$fit))  data$fit$x[na.ind$fit]   <- m[na.ind$fit[,"col"]]
-    if(!is.null(na.ind$test)) data$test$x[na.ind$test] <- m[na.ind$test[,"col"]]
-
-    impute_failed <- intersect(which(is.na(m)), na.feats)
-    if(any(impute_failed)){
-        data <- pre_remove(data, impute_failed)
-        warning(sprintf("Could not impute %i features.", length(impute_failed)))
-    }
-    data
-}
-#' @rdname pre_impute
-#' @export
-pre_impute_median <- function(data){
-    pre_impute(data, fun=median, na.rm=TRUE)
-}
-#' @rdname pre_impute
-#' @export
-pre_impute_mean <- function(data){
-    pre_impute(data, fun=mean, na.rm=TRUE)
-}
-
-#' Nearest neighbors imputation
+#' Factors will be converted to one logical column per level (or one fewer if a
+#' base level is specified).
 #' 
-#' Nearest neighbor methods needs to have a distance matrix of the dataset it works on.
-#' When doing repeated model fittings on subsets of the entire dataset it is
-#' unnecessary to recalculate it every time, therefore this function requires
-#' the user to manually calculate it prior to resampling and supply it in a
-#' wrapper function.
-#'
-#' Features with fewer than \code{k} non-missing values will be removed
-#' automatically.
-#' 
-#' @param data Fitting and testing data sets, as returned by
-#'   \code{\link{pre_split}}.
-#' @param k Number of nearest neighbors to calculate mean from. Set to < 1 to
-#'   specify a fraction.
-#' @param distance_matrix A matrix, \code{\link{dist}} object or
-#'   \code{"auto"}. Notice that \code{"auto"} will recalculate the distance
-#'   matrix in each fold, which is only meaningful in case the features of
-#'   \code{x} vary between folds. Otherwise you are just wasting time.
-#'   
+#' @param data Pre-processed data set, as produced by \code{\link{pre_split}}.
+#' @param feature Character vector with names of features to convert.
+#'   Defaults to all factors in the data set.
+#' @param base Sent to \code{\link{factor_to_logical}}. To specify different bases for
+#'   different columns supply a vector or list with named elements.
+#' @param drop Sent to \code{\link{factor_to_logical}}. To specify different bases for
+#'   different columns supply a vector or list with named elements.
 #' @examples
-#' x <- iris[-5]
-#' x[sample(nrow(x), 30), 3] <- NA
-#' my.dist <- dist(x)
-#' evaluate(modeling_procedure("lda"), x = x, y = iris$Species,
-#'     pre_process = function(...){
-#'         pre_split(...) %>% pre_impute_knn(k = 4, distance_matrix = my.dist)
-#'     }
+#' x <- mtcars[-1]
+#' x <- transform(x,
+#'     cyl = factor(cyl, ordered=TRUE),
+#'     vs = factor(vs),
+#'     gear = factor(gear)
 #' )
+#' y <- mtcars$mpg
+#' cv <- resample("crossvalidation", y)
+#' data <- pre_split(x, y, cv[[1]]) %>%
+#'     pre_factor_to_logical(base = c(cyl="4", vs="0"), 
+#'                           drop=c(cyl=FALSE, gear=FALSE))
+#' data$fit$x
 #' @author Christofer \enc{Bäcklin}{Backlin}
 #' @export
-pre_impute_knn <- function(data, k=.05, distance_matrix){
-    na.ind <- na_index(data)
-    if(is.null(na.ind)) return(data)
-
-    if(k < 1) k <- max(1, round(.05*length(index_fit(data$fold))))
-    if(k > sum(data$fold > 0, na.rm=TRUE))
-        stop("k is larger than number of fitting observations.")
-
-    # Exclude features with fewer non-NAs than k
-    na.count <- na.ind %>%
-        filter_("set == 'fit'") %>%
-        group_by_("col") %>%
-        summarize_(n = "length(row)")
-    impute_failed <- nrow(data$fit$x) - na.count$n < k
- 
-    if(any(impute_failed)){
-        data <- pre_remove(data, impute_failed)
-        warning(sprintf("Could not knn-impute %i features.", 
-                        length(impute_failed)))
-        na.ind %<>% filter_(~!col %in% impute_failed)
-        na.ind$col <- na.ind$col - findInterval(na.ind$col, which(impute_failed))
+pre_factor_to_logical <- function(data, feature, base=1L, drop=TRUE){
+    stopifnot(is.data.frame(data$fit$x))
+    convert <- function(x, base, base_missing, drop, drop_missing, name, warn=TRUE){
+        withCallingHandlers({
+            if(name %in% feature){
+                factor_to_logical(x,
+                    base = if(base_missing) 1L else base,
+                    drop = if(drop_missing) TRUE else drop)
+            } else x
+        }, warning = function(w){
+            if(warn){
+                warning(sprintf("Column `%s` threw a warning when converted to logical: %s",
+                                name, w$message))
+            }
+            invokeRestart("muffleWarning")
+        }, error = function(e){
+            stop(sprintf("Column `%s` could not be converted to logical: %s",
+                         name, e$message))
+        })
     }
-
-    # Check that the distance matrix is in order
-    if(missing(distance_matrix))
-        stop("You must supply a distance matrix, see `?pre_impute_knn` for details.")
-    if(is.character(distance_matrix) && distance_matrix == "auto"){
-        distance_matrix <- matrix(nrow = length(data$fold), ncol = length(data$fold))
-        ind <- c(index_fit(data$fold, allow_oversample=FALSE),
-                 index_test(data$fold))
-        distance_matrix[ind, ind] <- as.matrix(dist(rbind(data$fit$x, data$test$x)))
-    } else if(!is.matrix(distance_matrix)){
-        distance_matrix <- as.matrix(distance_matrix)
+    factor_feature <- colnames(data$fit$x)[sapply(data$fit$x, is.factor)]
+    if(missing(feature))
+        feature <- factor_feature
+    if(any(!feature %in% factor_feature))
+        stop(sprintf("%s are not factors in the data set.",
+             example_string("feature", feature[!feature %in% factor_feature])))
+    columns <- Map(convert, data$fit$x,
+                   base[colnames(data$fit$x)], !colnames(data$fit$x) %in% names(base),
+                   drop[colnames(data$fit$x)], !colnames(data$fit$x) %in% names(drop),
+                   colnames(data$fit$x), MoreArgs=list(warn=TRUE))
+    data$feature_selection <- rep(data$feature_selection,
+        vapply(columns, function(x) if(!is.null(ncol(x))) ncol(x) else 1L, integer(1)))
+    data$fit$x <- do.call(cbind, columns)
+    names(data$feature_selection) <- colnames(data$fit$x)
+    rm(columns)
+    if(nrow(data$test$x) > 0){
+        data$test$x <- do.call(cbind, 
+            Map(convert, data$test$x,
+                base[colnames(data$test$x)], !colnames(data$test$x) %in% names(base),
+                drop[colnames(data$test$x)], !colnames(data$test$x) %in% names(drop),
+                colnames(data$test$x), MoreArgs=list(warn=FALSE))
+        )
+    } else {
+        data$test$x <- data$fit$x[FALSE,]
     }
-    if(any(length(data$fold) != dim(distance_matrix)))
-        stop("Distance matrix does not match dataset.")
-
-    # Perform the imputation
-    diag(distance_matrix) <- NA
-    NN <- as.data.frame(apply(distance_matrix[na.ind$row, index_fit(data$fold)], 1, order))
-    na.ind$fill <- mapply(function(i, col){
-        x <- data$fit$x[i, col]
-        mean(x[!is.na(x)][1:k])
-    }, NN, na.ind$col)
-
-    if(any(na.ind$set == "fit"))
-        data$fit$x[as.matrix(na.ind[na.ind$set == "fit", c("row", "col")])] <-
-            na.ind$fill[na.ind$set == "fit"]
-    if(any(na.ind$set == "test"))
-        data$test$x[as.matrix(na.ind[na.ind$set == "test", c("row", "col")])] <-
-            na.ind$fill[na.ind$set == "test"]
+    stopifnot(identical(colnames(data$fit$x), colnames(data$test$x)))
     data
 }
 
-#' Regular imputation
-#'
-#' If you want to impute, build model and predict you should use
-#' \code{\link{pre_impute_median}} or \code{\link{pre_impute_knn}}.
-#' This function imputes using all observations
-#' without caring about cross-validation folds.
-#'
-#' For additional information on the parameters see \code{\link{pre_impute_knn}}
-#' and \code{\link{pre_impute}}.
+#' Print log message during pre-processing
 #' 
-#' @param x Dataset.
-#' @param k Number of nearest neighbors to use.
-#' @param distance_matrix Distance matrix.
-#' @return An imputed matrix.
-#' @examples
-#' x <- matrix(rnorm(36), 6, 6)
-#' x[sample(length(x), 5)] <- NA
+#' @param data Pre-processed data set.
+#' @param ... Sent to \code{\link{log_message}}
+#' @return The same data set as inputted. This only purpose of this function is
+#'   to print a log message as a side effect.
+#' @seealso \code{\link{pre_process}}, \code{\link{log_message}}.
 #' @author Christofer \enc{Bäcklin}{Backlin}
-#' @name impute
-#' @seealso \code{\link{emil}}, \code{\link{pre_process}},
-#'   \code{\link{pre_impute_knn}}, \code{\link{pre_impute_median}}
-{}
-#' @examples
-#' impute_knn(x)
-#' @rdname impute
 #' @export
-impute_knn <- function(x, k=.05, distance_matrix="auto"){
-    if(identical(distance_matrix, "auto"))
-        distance_matrix <- dist(x)
-    pre_split(x, y=NULL) %>%
-        pre_impute_knn(k=k, distance_matrix=distance_matrix) %>%
-        (function(data) data$fit$x)
-}
-#' @examples
-#' impute_median(x)
-#' @rdname impute
-#' @export
-impute_median <- function(x){
-    pre_split(x, y=NULL) %>%
-        pre_impute_median %>%
-        (function(data) data$fit$x)
+pre_log_message <- function(data, ...){
+    log_message(...)
+    data
 }
 
-#' Print method for pre-processed data
-#' 
-#' @method print preprocessed_data
-#' @param x Pre-processed data, as produced by \code{\link{pre_split}}.
-#' @param ... Ignored, kept for S3 consistency.
-#' @return Nothing
-#' @author Christofer \enc{Bäcklin}{Backlin}
-#' @export
-print.preprocessed_data <- function(x, ...){
-    feature <- table(factor(x$feature_selection, c(FALSE, TRUE)))
-    cat("Pre-processed data set `", x$name, "` of ",
-        feature[2], " features",
-        if(feature[1] > 0) sprintf("(%i removed).\n", feature[1]) else ".\n",
-        nrow(x$fit$x), " observations for model fitting,\n",
-        nrow(x$test$x), " observations for model evaluation.\n", sep="")
-}
